@@ -3,12 +3,13 @@ use std::io::{BufReader, Cursor, Read};
 use std::path::PathBuf;
 
 use calamine::{open_workbook_auto, open_workbook_auto_from_rs, Error, Reader, Sheets};
+use pyo3::exceptions::PyTypeError;
 use pyo3::prelude::*;
 use pyo3::types::{PyString, PyType};
 use pyo3_file::PyFileLikeObject;
 
 use crate::utils::err_to_py;
-use crate::{CalamineError, CalamineSheet, SheetMetadata};
+use crate::{CalamineSheet, SheetMetadata, WorksheetNotFound};
 
 enum SheetsEnum {
     File(Sheets<BufReader<File>>),
@@ -33,25 +34,12 @@ impl SheetsEnum {
         }
     }
 
-    fn worksheet_range(
-        &mut self,
-        name: &str,
-    ) -> Option<Result<calamine::Range<calamine::DataType>, Error>> {
+    fn worksheet_range(&mut self, name: &str) -> Result<calamine::Range<calamine::Data>, Error> {
         match self {
             SheetsEnum::File(f) => f.worksheet_range(name),
             SheetsEnum::FileLike(f) => f.worksheet_range(name),
         }
     }
-
-    fn worksheet_range_at(
-        &mut self,
-        index: usize,
-    ) -> Option<Result<calamine::Range<calamine::DataType>, Error>> {
-        match self {
-            SheetsEnum::File(f) => f.worksheet_range_at(index),
-            SheetsEnum::FileLike(f) => f.worksheet_range_at(index),
-        }
-    }
 }
 
 #[pyclass]
@@ -76,59 +64,56 @@ impl CalamineWorkbook {
 
     #[classmethod]
     #[pyo3(name = "from_object")]
-    fn py_from_object(_cls: &PyType, path_or_filelike: PyObject) -> PyResult<Self> {
-        Self::from_object(path_or_filelike)
+    fn py_from_object(_cls: &PyType, py: Python<'_>, path_or_filelike: PyObject) -> PyResult<Self> {
+        Self::from_object(py, path_or_filelike)
     }
 
     #[classmethod]
     #[pyo3(name = "from_filelike")]
-    fn py_from_filelike(_cls: &PyType, filelike: PyObject) -> PyResult<Self> {
-        Self::from_filelike(filelike)
+    fn py_from_filelike(_cls: &PyType, py: Python<'_>, filelike: PyObject) -> PyResult<Self> {
+        py.allow_threads(|| Self::from_filelike(filelike))
     }
 
     #[classmethod]
     #[pyo3(name = "from_path")]
-    fn py_from_path(_cls: &PyType, path: &str) -> PyResult<Self> {
-        Self::from_path(path)
+    fn py_from_path(_cls: &PyType, py: Python<'_>, path: PyObject) -> PyResult<Self> {
+        if let Ok(string_ref) = path.downcast::<PyString>(py) {
+            let path = string_ref.to_string_lossy().to_string();
+            return py.allow_threads(|| Self::from_path(&path));
+        }
+
+        if let Ok(string_ref) = path.extract::<PathBuf>(py) {
+            let path = string_ref.to_string_lossy().to_string();
+            return py.allow_threads(|| Self::from_path(&path));
+        }
+
+        Err(PyTypeError::new_err(""))
     }
 
-    fn get_sheet_by_name(&mut self, name: &str) -> PyResult<CalamineSheet> {
-        let range = self
-            .sheets
-            .worksheet_range(name)
-            .unwrap_or_else(|| Err(Error::Msg("Workbook is empty")))
-            .map_err(err_to_py)?;
-        Ok(CalamineSheet::new(name.to_owned(), range))
+    #[pyo3(name = "get_sheet_by_name")]
+    fn py_get_sheet_by_name(&mut self, py: Python<'_>, name: &str) -> PyResult<CalamineSheet> {
+        py.allow_threads(|| self.get_sheet_by_name(name))
     }
 
-    fn get_sheet_by_index(&mut self, index: usize) -> PyResult<CalamineSheet> {
-        let name = self
-            .sheet_names
-            .get(index)
-            .ok_or_else(|| CalamineError::new_err("Workbook is empty"))?
-            .to_string();
-        let range = self
-            .sheets
-            .worksheet_range_at(index)
-            .unwrap_or_else(|| Err(Error::Msg("Workbook is empty")))
-            .map_err(err_to_py)?;
-        Ok(CalamineSheet::new(name, range))
+    #[pyo3(name = "get_sheet_by_index")]
+    fn py_get_sheet_by_index(&mut self, py: Python<'_>, index: usize) -> PyResult<CalamineSheet> {
+        py.allow_threads(|| self.get_sheet_by_index(index))
     }
 }
 
 impl CalamineWorkbook {
-    pub fn from_object(path_or_filelike: PyObject) -> PyResult<Self> {
-        Python::with_gil(|py| {
-            if let Ok(string_ref) = path_or_filelike.downcast::<PyString>(py) {
-                return Self::from_path(string_ref.to_string_lossy().to_string().as_str());
-            }
+    pub fn from_object(py: Python<'_>, path_or_filelike: PyObject) -> PyResult<Self> {
+        if let Ok(string_ref) = path_or_filelike.downcast::<PyString>(py) {
+            let path = string_ref.to_string_lossy().to_string();
+            return py.allow_threads(|| Self::from_path(&path));
+        }
 
-            if let Ok(string_ref) = path_or_filelike.extract::<PathBuf>(py) {
-                return Self::from_path(string_ref.to_string_lossy().to_string().as_str());
-            }
+        if let Ok(string_ref) = path_or_filelike.extract::<PathBuf>(py) {
+            let path = string_ref.to_string_lossy().to_string();
+            return py.allow_threads(|| Self::from_path(&path));
+        }
 
-            Self::from_filelike(path_or_filelike)
-        })
+        py.allow_threads(|| Self::from_filelike(path_or_filelike))
     }
 
     pub fn from_filelike(filelike: PyObject) -> PyResult<Self> {
@@ -159,4 +144,18 @@ impl CalamineWorkbook {
             sheet_names,
         })
     }
+
+    fn get_sheet_by_name(&mut self, name: &str) -> PyResult<CalamineSheet> {
+        let range = self.sheets.worksheet_range(name).map_err(err_to_py)?;
+        Ok(CalamineSheet::new(name.to_owned(), range))
+    }
+
+    fn get_sheet_by_index(&mut self, index: usize) -> PyResult<CalamineSheet> {
+        let name = self
+            .sheet_names
+            .get(index)
+            .ok_or_else(|| WorksheetNotFound::new_err(format!("Worksheet '{}' not found", index)))?
+            .to_string();
+        self.get_sheet_by_name(&name)
+    }
 }
